/*
 * taken from the Java Developers Journal...
 *
 */

import java.io.*;
import java.net.*;

public class DownloadInputStream extends InputStream 
{
  private InputStream internalInputStream;
  private long length = -1;
  private long startTime = -1;
  private long bytesRead = 0;
  
  public DownloadInputStream(InputStream is, long length) 
  {
    internalInputStream = is;
    this.length = length;
  }
  
  public DownloadInputStream(InputStream is) 
  {
    internalInputStream = is;
  }
  
  public DownloadInputStream(URL url) throws IOException 
  {
    URLConnection connection = url.openConnection();
    this.length = connection.getContentLength();
    
    internalInputStream = connection.getInputStream();
  }
  
   public float getDownloadRate() 
  {
    if (startTime != -1) 
      {
	long currentTime = System.currentTimeMillis();
	
	return bytesRead / (float)(currentTime - startTime);
      }
    else
      return -1.0f;
  }
  
  public long getDownloadTime() 
  {
    if (startTime != -1)
      return System.currentTimeMillis() - startTime;
    else
      return 0;
  }
  
  public float getDownloadedPercent() 
  {
//      System.out.println("startTime = " + startTime + "   length = " + length );
    if (startTime == -1 || length == -1)
      return -1.0f;
    else 
      {
	return (float)bytesRead / (float)length;
      }
  }
  
  public long getBytesRead() 
  {
    return bytesRead;
  }
  
  public long getBytesTotal() 
  {
    return length;
  }
  
  public long getEstimatedDownloadTime() 
  {
    if (startTime == -1 || length == -1)
      return -1;
    
    long currentTime = System.currentTimeMillis();
    
    return (long)((length - bytesRead) / getDownloadRate());
  }
  
  public int available() throws IOException 
  {
    return internalInputStream.available();
  }
  
  public void close() throws IOException 
  {
    internalInputStream.close();
  }
  
  public void mark(int readlimit) 
  {
    internalInputStream.mark(readlimit);
  }
  
  public boolean markSupported() 
  {
    return internalInputStream.markSupported();
  }
  
  public int read() throws IOException 
  {
    if (startTime == -1)
      startTime = System.currentTimeMillis();
    
    int b = internalInputStream.read();
    
    if (b != -1)
      bytesRead++;
    
    return b;
  }
  
  public int read( byte[] b ) throws IOException
  {
    if( startTime == -1 )
      startTime = System.currentTimeMillis();
    
    int read = internalInputStream.read( b );
    bytesRead += read;
    return read;
  }

  public int read(byte[] b, int off, int len) throws IOException 
  {
    if (startTime == -1)
      startTime = System.currentTimeMillis();
    
    int read = internalInputStream.read(b,off,len);
    bytesRead += read;
    return read;
  }
  
  public void reset() throws IOException 
  {
    startTime = -1;
    bytesRead = 0;
    
    internalInputStream.reset();
  }
  
  public long skip(long n) throws IOException 
  {
    long skipped = internalInputStream.skip(n);
    bytesRead += skipped;
    
    return skipped;
  }
}
