/*
    Copyright (C) 2000  DMark W. Webb (markwebb@adelphia.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



import java.io.*;
import java.util.*;
import java.net.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class DownloadThread extends JDialog implements Runnable
{
  private Thread downloadThread = null;
  private boolean running = true;
  private Hashtable hashtable;
  private FileInputStream fis;
  private String output;
  private String filename;
  private long size;
  private String ip;
  private int port;
  private JProgressBar bar;
  private JLabel lTransfertRate, lEstimatedDownloadTime, lBytesRead, lBytesTotal;
  private DownloadInputStream dis;
  InputStream in;
  OutputStream out;
  Socket socket;

  public DownloadThread( Frame owner, String output, String filename, String size, String ip, String port )
  {
    super(owner, "Download Status", false);
    
    this.output = output;    
    this.filename = filename;
    this.size = Long.parseLong(size);
    this.ip = ip;
    this.port = Integer.parseInt(port);

    JLabel lFile = new JLabel( "File: " + filename );
    lBytesRead = new JLabel( "Bytes read:" );
    lBytesTotal = new JLabel( "Size:" );
    lTransfertRate = new JLabel( "Transfer rate:" );
    lEstimatedDownloadTime = new JLabel( "Estimated downloading time:" );
    bar = new JProgressBar( JProgressBar.HORIZONTAL, 0, 100 );
    
    getContentPane().setLayout( new GridLayout(0, 1, 5, 5) );
    
    getContentPane().add(lFile);
    getContentPane().add(lBytesRead);
    getContentPane().add(lBytesTotal);
    getContentPane().add(lTransfertRate);
    getContentPane().add(lEstimatedDownloadTime);
    getContentPane().add(bar);

    addWindowListener(new WindowAdapter() {
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowClosing(WindowEvent e) 
	{
	  stop();
	  setVisible( false );
	}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {}
	public void windowOpened(WindowEvent e) {}});
    
    pack();
    show();
  }
  
  public synchronized void start()
  {
    if( downloadThread == null )
      downloadThread = new Thread( this );
    downloadThread.start();
  }

  public synchronized void stop()
  {
    System.out.println("The download was stopped.");
    MainWindow.getTransferPanel().getDataModel().remove( filename );
    running = false;
    downloadThread = null;
  }

  public synchronized void run()
  {
    while( running )
      download();
  }

  public void download()
  {
    try{
      Socket socket = new Socket(InetAddress.getByName(ip), new Integer(port).intValue());
      socket.setSoTimeout(30*1000);
      
      in = socket.getInputStream();
      out = socket.getOutputStream();
      dis = new DownloadInputStream( in, size );
      
      out.write( output.getBytes());
      
      byte [] b = new byte [1024 * 5];
      int count;
      StringBuffer buf = new StringBuffer();
      OutputStream saveStream = System.out;
      
      long byteCount = 0;
      long iterCount = 0;
      long start = System.currentTimeMillis();
      long begin = start;
      boolean saving = false;
      String msg = "";
      
      File downloadfile = new File( System.getProperty( "user.home" ) + 
				    System.getProperty( "file.separator" ) + 
				    filename );
      try{
	while((count = dis.read(b)) != -1)
	  {
	    if(saving)
	      {
		byteCount += count;
		iterCount += count;
		start = System.currentTimeMillis();
	      }

	    String str = new String(b, 0, count);
	    
	    //
	    // this is what writes to the file....
	    //
	    saveStream.write(b, 0, count);
	    
	    lBytesRead.setText( "Bytes read:" + dis.getBytesRead() );
	    lBytesTotal.setText( "Size:" + size );
	    lTransfertRate.setText( "Transfer rate:" + dis.getDownloadRate() * (1000f/1024f) + " kb/s" );
	    lEstimatedDownloadTime.setText( "Estimated downlaoding time:" + (int)(dis.getEstimatedDownloadTime()
										  / 1000) + " sec." );
	    bar.setValue( (int)(dis.getDownloadedPercent() * 100) );
	    
//  	    System.out.println("download thread: percent done = " + (int)(dis.getDownloadedPercent()));
	    
	    if(saving == false)
	      {
		buf.append(str);
	      }
//  	    else
//  	      {
//  		// check to see if we have gotten the entire file
//  		if(byteCount >= downloadfile.length())
//  		  break;
//  	      }
	    
	    if(saving == false && buf.toString().endsWith("\r\n\r\n"))
	      {
		String resp = buf.toString();
		int ind = resp.indexOf("STP/1.0");
		
		if(ind == -1)
		  throw new IOException("Did not receive " + "required \"STP/1.0\" response");
		
		resp = resp.substring(ind);
		resp = resp.substring(0, resp.indexOf("\r\n"));
		
		if(resp.indexOf("OK") == -1)
		  throw new IOException("Request error:(" + resp + ")");
		
		saveStream = new BufferedOutputStream(new FileOutputStream(downloadfile.getAbsolutePath(), true));
		saving = true;
	      }
	  }
      }catch(InterruptedIOException timeout){
//  	System.err.println("");
//  	System.err.println("Download timed out");
//  	System.out.println("\texception -- " + timeout );
	this.setVisible( false );
      }catch(Exception e){
	System.err.println("");
	System.err.println("Exception reported in DownloadThread");
	System.out.println("\texception -- " + e );
	this.setVisible( false );
      }
      
      
      saveStream.close();
      
      System.err.println("");
      System.err.println("Done");
      this.setVisible( false );
    }catch(Exception e){
      System.out.println("in DownloadThread: " + e );
      this.setVisible( false );
    } 
    
    stop();
    
  }
}
