/*
    Copyright (C) 2000  DMark W. Webb (markwebb@adelphia.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



import java.awt.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.util.Vector;
import java.util.Enumeration;

class HotListDataModel extends AbstractTableModel
{
  String[] columns = { "Filename", "Size", "Path", "IP", "Port" };

  boolean[] columnsOnOff = new boolean[columns.length];
  Vector filename = new Vector();
  Vector size = new Vector();
  Vector path = new Vector();
  Vector ip = new Vector();
  Vector port = new Vector();
  Vector entry = new Vector( columns.length );

  public HotListDataModel()
  {
    super();
    for(int i=0; i<columns.length; i++)
      columnsOnOff[i]  = true ;
  
    entry.add( filename );
    entry.add( size );
    entry.add( path );
    entry.add( ip );
    entry.add( port );
  }
  
  public int getColumnCount() 
  {
    int count = 0 ;
    for( int i=0; i<columns.length; i++ )
      if( columnsOnOff[i] == true )
        count++ ;
    return( count ) ;
  }
  
  public Object getValueAt( int row, int column )
  {
    return( ((Vector)(entry.elementAt(column))).elementAt(row) ) ;
  }
  
  public void setValueAt( Object value, int row, int column )
  {
    Vector vector = (Vector)entry.get( column );
    try{
      vector.setElementAt( value, row );
    }catch(ArrayIndexOutOfBoundsException aioobe){  
      System.out.println("There was a problem setting the value for the contact list");
    }
    fireTableChanged( new TableModelEvent( this, row ) );
  }
  
  public void remove( int row )
  {
    filename.remove( row );
    size.remove( row );
    path.remove( row );
    ip.remove( row );
    port.remove( row );
    fireTableChanged( new TableModelEvent( this, 0, getRowCount(), 
                                           TableModelEvent.ALL_COLUMNS, row ) );
  }
  
  public void addRow( SearchResult result )
  {
    filename.add( (String)result.get( "Filename" ) );
    size.add( (String)result.get( "Size") );
    path.add( (String)result.get( "Path" ));
    ip.add( (String)result.get("IP"));
    port.add( (String)result.get("Port"));
    
    fireTableChanged( new TableModelEvent( this, filename.size(), filename.size(), 
                                           TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT ) ) ;
  }
  
  public int getRowCount()                    { return ( filename.size() ); }
  public Class getColumnClass( int c )        { return ( getValueAt( 0, c ).getClass() ); }

  public String getFilenameAt( int row ){ return (String)filename.elementAt(row); }
  public String getSizeAt( int row ){ return (String)size.elementAt(row); }
  public String getPathAt( int row ){ return (String)path.elementAt(row); }
  public String getIpAt( int row ){ return (String)ip.elementAt(row); }
  public String getPortAt( int row ){ return (String)port.elementAt(row); }
  
  public boolean isCellEditable( int row, int column )
  {
    return( false ) ;
  }
  
  public String getColumnName( int columnIndex )
  {
    int count = 0 ;
    int i ;
    for( i=0; i<columns.length; i++ )
      {
        if( columnsOnOff[i] == true )
          count++ ;
        if( count == (columnIndex+1) )
          break ;
      }
    return( columns[i] ) ;
  } 
}
