/*
    Copyright (C) 2000  DMark W. Webb (markwebb@adelphia.net)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/



import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class SearchPanel extends JPanel implements ActionListener, MouseListener
{
  JTextField searchField;
  JComboBox mediaType, communityType, numberofresults, minconnection, duration;
  JButton find, clear, download, cancel;
  JTable searchtable;
  SearchDataModel sdm;
  JPopupMenu popupMenu;
  JMenuItem adduser, downld;
  Download downloadthread;
  
  public SearchPanel()
  {
    super();
    this.setLayout( new BorderLayout() );
    
    popupMenu = new JPopupMenu();
    adduser = new JMenuItem( "Add user to hotlist" );
    adduser.addActionListener( this );
    popupMenu.add( adduser );
    downld = new JMenuItem( "Download file" );
    downld.addActionListener( this );
    popupMenu.add( downld );

    JPanel top = new JPanel();
    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    top.setLayout( gbl );
    c.fill = GridBagConstraints.BOTH;
    c.insets = new Insets( 2, 5, 2, 5 );
    c.weightx = 1.0;
    
    c.gridwidth = 1;
    top.add( new JLabel( "Search For:" ), c);
    top.add( new JLabel( "Media Type:" ), c);
    c.gridwidth = GridBagConstraints.REMAINDER;
    top.add( new JLabel( "Look:" ), c );

    c.gridwidth = 1;
    top.add( searchField = new JTextField(15), c);
    String[] media = { "any", "audio", "image", "video", "-----", "asf", 
                       "avi", "bmp", "gif", "jpeg", "jpg", "mid", "midi", 
                       "mov", "mp2", "mp3", "mpeg", "mpg", "qt", "rm", "ra",
                       "viv", "vivo", "vob", "vqf", "wav", "wma" };
    mediaType = new JComboBox( media );
    top.add( mediaType, c );
    String[] community = { "Community", "Web" };
    communityType = new JComboBox( community );
    top.add( communityType, c );
    c.gridwidth = GridBagConstraints.REMAINDER;
    find = new JButton( "Find" );
    find.addActionListener( this );
    top.add( find, c );
    
    c.gridwidth = 1;
    top.add( new JLabel( "Max Results" ), c);
    top.add( new JLabel( "Min. Connection" ), c);
    c.gridwidth = GridBagConstraints.REMAINDER;
    top.add( new JLabel( "Duration" ), c );

    c.gridwidth = 1;
    String[] results = { "10", "20", "50", "100", "150", "200", "250", "300" };
    numberofresults = new JComboBox( results );
    top.add( numberofresults, c);
    String[] connection = { "14.4K", "28.8K", "33.6K", "56K", "64K", "128K",
                            "Cable", "DSL", "T1", "T3", "Any" };
    minconnection = new JComboBox( connection );
    top.add( minconnection, c );
    String[] dur = { "1", "2", "3", "5", "10", "15", "20", "30" };
    duration = new JComboBox( dur );
    top.add( duration, c );
    c.gridwidth = GridBagConstraints.REMAINDER;
    clear = new JButton( "Clear" );
    top.add( clear, c );
    
    add( top, "North" );    

    JPanel middle = new JPanel();
    sdm = new SearchDataModel();  
    searchtable = new JTable( sdm );
    searchtable.addMouseListener( this );
    searchtable.setPreferredScrollableViewportSize(new Dimension(800, 350));
    JScrollPane scrollpane = new JScrollPane( searchtable, JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
                                              JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS );
    middle.add( scrollpane, BorderLayout.CENTER );

    add( middle, "Center" );

    JPanel bottom = new JPanel();
    download = new JButton( "Download" );
    download.addActionListener( this );
    bottom.add( download );
    cancel = new JButton( "Cancel" );
    cancel.addActionListener( this );
    cancel.setEnabled( false );
    bottom.add( cancel );
    add( bottom, "South" );
  }

  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mousePressed(MouseEvent e) {  maybeShowPopup(e); }
  public void mouseReleased(MouseEvent e) { maybeShowPopup(e); }
  
  private void maybeShowPopup(MouseEvent e) 
  {
    if( searchtable.getSelectedRow() == -1 )
      return;
    
    if( e.isPopupTrigger() ) 
      popupMenu.show(e.getComponent(), e.getX(), e.getY());
  }

  private String getSeparator( String test )
  {
    if( test.indexOf("/") > 0 )
      return "/";
    else
      return "\\";
  }

  public void actionPerformed( ActionEvent e )
  {
    System.out.println("SearchPanel: " + e.getActionCommand());
    if( e.getSource() == find )
      {
//      for( int i=0; i<sdm.getRowCount(); i++ )
//        sdm.remove( i );
        
        int time = Integer.parseInt((String)duration.getSelectedItem());
        Search search = new Search( );
        search.getResults(sdm, (String)mediaType.getSelectedItem(), (String)numberofresults.getSelectedItem(),
                          minconnection.getSelectedIndex(), time*60, searchField.getText() );
      }
    else if( e.getSource() == download )
      {
        int row = searchtable.getSelectedRow();
        if( row == -1 )
          return;
        
        String filename = sdm.getFilenameAt( row );
        System.out.println("SearchPanel: downloading " + filename );
	String path = sdm.getPathAt(row);
	String file = sdm.getFilenameAt(row);
	String searchfilename = null;
	if( path.endsWith("/") || path.endsWith("\\"))
	  searchfilename = path + file;
	else 
	  searchfilename = path + getSeparator(path) + file;
	
		
  	TransferPanelDataModel transfermodel = MainWindow.getTransferPanel().getDataModel();
	transfermodel.addRow(  filename, sdm.getSizeAt(row), "Downloading", "progress", 
  			       "transfer", "rate", sdm.getUsernameAt(row), sdm.getSpeedAt(row), "not much" );
	
	MainWindow.showPanel( MainWindow.TRANSFER );
	cancel.setEnabled( true );
	downloadthread = new Download(sdm.getUsernameAt(row), searchfilename, sdm.getSizeAt(row),
				      sdm.getSpeedAt(row), sdm.getIpAt(row), sdm.getPortAt(row) );
      }
    else if( e.getSource() == downld )
      {
        download.doClick();
      }
    else if( e.getSource() == adduser )
      {
        int row = searchtable.getSelectedRow();
        MainWindow.getHotListPanel().addUser( sdm.getUsernameAt( row ));
        MainWindow.showPanel( MainWindow.HOTLIST );
      }
    else if( e.getSource() == cancel )
      {
	if( searchtable.getSelectedRow() == -1 )
	  return;
	
	downloadthread.stop();
	cancel.setEnabled( false );
      }
  }
}

